/*  -----------------------------------------------------------------------
    Program  :  GetALighter_DarkerColor.sas

    Author   :  Perry Watts
    Date     :  Friday, January 21, 2011
    
    Project  :  ODS Styles Paper for SGF with 9.2 SAS

    Purpose  :  Translate an RGB code to a lighter or darker shade to
                change background or foreground colors in ODS style
                definitions. This macro is a macro function that calls
                subordinate macro functions (in this library). Program flow
                is as follows:
                1) Translate CX002288 to Decimal RGB            -- %RGBDEC
                2) Translate decimal RGB to decimal HLS         -- %RGBtoHLS
                4) Change the value of Lightness (in decimal)   -- PCTLIGHTNESS parm
                5) Translate the result back to RGB Decimal.    -- %HLSTORGB (nested)
                6) Convert to RGB Hex to get CXCDD9FF           -- %RGBHEX
    Input    :  RGB Hexadecimal code
    Output   :  A lighter shade of the Original hexadecimal code.
    Notes    :  *RGB=Red|Green|Blue
                *HLS=Hue|Lightness|Saturation -> need the LIGHTNESS component
                 to do the calculations.
                *The subordinate macros that are called from this routine pre-date
                 SAS Institute's color utility macros.  I still prefer mine, because
                 there are no percent restrictions to them, and they do not have
                 any problems with round-trip conversions between RGB and HLS 
                 alluded to in the SAS/GRAPH 9.1 manual (p. 103). (I have tested).
                *To find out more about working with macro libraries, see
                 Art Carpenter's macro book.
                *Replace my subdirectory structure with yours!!              
    ------------------------------------------------------------------- */
   options nodate number pageno=1 fmtsearch=(work) mautosource nomprint;
   options sasautos=(sasautos 'C:\ETC');
  %macro getALighter_DarkerColor(RGBin=, pctLightness=);
    %local RGBdecIn HLSdecIn hue sat;
    %let RGBdecIn = %RGBDec(&RGBin);
    %let HLSdecIn = %RGBtoHLS(&RGBdecIn);
  /* GET RID OF THE COMMA DELIMITERS IN HLSDECIN */
    %let HLSdecIn=%sysfunc(compress("&HLSdecIn",",")); 
    %let hue = %substr(&hlsdecin,1,3);
    %let oldLightCode=%substr(&hlsdecin,4,3);
    %let sat = %substr(&hlsdecin,7);

    %let pctLightness=%sysevalf(&pctLightness/100.0);
    %let newLightCode=%sysevalf(&pctLightness * 255);
    %let newLightCode=%sysfunc(round(&newLightCode));

    %let HLSdecOut=&hue.,&newLightCode.,&sat.;
    %let RGBhexOut=%rgbhex(%hlstoRGB(&hlsDecOut));
    &RGBhexOut;
  %mend getALighter_DarkerColor;

  /* Example macro invocations follow.  They could be moved to a calling program. */
  %let RGB90 = %getALighter_DarkerColor(RGBin=cx002288, pctLightness=90);
  %put RGB90 = &RGB90;

  %let RGB95 = %getALighter_DarkerColor(RGBin=cx002288, pctLightness=95);
  %put RGB95 = &RGB95;

  %let RGB97 = %getALighter_DarkerColor(RGBin=cx002288, pctLightness=97);
  %put RGB97 = &RGB97;

  %let RGB80  = %getALighter_DarkerColor(RGBin=cx002288, pctLightness=80);
  %put RGB80 = &RGB80;

  %let RGB50  = %getALighter_DarkerColor(RGBin=cx002288, pctLightness=50);
  %put RGB50 = &RGB50;

  %let RGB30  = %getALighter_DarkerColor(RGBin=cx002288, pctLightness=30);
  %put RGB30 = &RGB30;

  %let RGB40  = %getALighter_DarkerColor(RGBin=cx002288, pctLightness=40);
  %put RGB40 = &RGB40;


  /* *****
       OUTPUT:
  RGB90 = CXCDD9FF
  RGB95 = CXE5EBFF
  RGB97 = CXEFF3FF
  RGB80 = CX99B3FF
  RGB50 = CX0140FF
  RGB30 = CX00269A
  RGB40 = CX0033CC
    ***** */
